@@ -372,19 +372,55 @@ def get_payments(user, password, stop=None):
     return payments
 
 
-def sessions_transforms(sessions, ramps, access_path, lanes_path):
+def get_access(user, password, cards):
+    access = dict()
+
+    url = 'http://mgrvde1/AbacusWebService/ServiceOperation.asmx'
+
+    # Get Card Access Data by Carpark
+    for card in cards:
+        xml = requests.get(url + '/getCardByCarrier' +
+                           '?user=' + user +
+                           '&pwd=' + password +
+                           '&cardCarrierNr=' + card)
+
+        access_data = ET.fromstring(xml.content)
+
+        if len(access_data) > 0:
+
+            ramp_id = access_data[0].get('ID')
+
+            if ramp_id != '0':
+                access_dict = {'EMI0' + access_data.get('ID'): access_data.get('Name').replace(card + ' ', '')}
+
+                access.update(access_dict)
+
+
+    return access
+
+
+def sessions_transforms(sessions, ramps, access_groups, devices):
     # Format Carpark Dictionary
     carparks = {}
     for line in ramps:
         carparks[line['Number']] = line['Name']
 
-    # Format Access Group Dictionary
-    with open(access_path, 'r') as file:
-        groups = ast.literal_eval(file.read())
+    if isinstance(access_groups, str):
+        # Format Access Group Dictionary
+        with open(access_groups, 'r') as file:
+            groups = ast.literal_eval(file.read())
+    else:
+        groups = access_groups
 
-    # Format Lanes Dictionary
-    with open(lanes_path, 'r') as file:
-        lanes = ast.literal_eval(file.read())
+    if isinstance(devices, str):
+        # Format Lanes Dictionary
+        with open(devices, 'r') as file:
+            lanes = ast.literal_eval(file.read())
+    else:
+        # Format Lanes Dictionary
+        lanes = dict()
+        for line in devices:
+            lanes[line['ID']] = line['Name']
 
     # SESSION ID
     sessions['Session ID'] = 'DE-' + sessions['payment_number'].astype(str)
@@ -396,8 +432,12 @@ def sessions_transforms(sessions, ramps, access_path, lanes_path):
     sessions['Lot Name'] = sessions['Qtg_ph_num'].map(carparks)
 
     # ACCESS GROUP
-    sessions['Access Group'] = pd.to_numeric(sessions['Qtg_tcc_num']).astype('Int64').map(groups)
-    sessions.loc[sessions['Access Group'].isna(), 'Access Group'] = 'Transient'
+    # ACCESS GROUPS
+    sessions['Access Group'] = sessions['Qtg_tic_iso'].map(groups)
+
+    sessions.loc[~sessions['Qtg_tic_iso'].str.contains('EMI', na=False), 'Access Group'] = 'Transient'
+
+    sessions.loc[sessions['Access Group'].isna(), 'Access Group'] = None
 
     # LANE
     sessions['Lane'] = pd.to_numeric(sessions['Qtg_tcc_num']).astype('Int64').map(lanes)
@@ -494,20 +534,33 @@ def sessions_transforms(sessions, ramps, access_path, lanes_path):
     return sessions
 
 
-def entries_transforms(entries, ramps, access_path, lanes_path):
+def entries_transforms(entries, ramps, access_groups, devices):
+    # Format Carpark Dictionary
     # Format Carpark Dictionary
     carparks = {}
     for line in ramps:
         carparks[line['Number']] = line['Name']
 
-    # Format Access Group Dictionary
-    with open(access_path, 'r') as file:
-        groups = ast.literal_eval(file.read())
+    if isinstance(access_groups, str):
+        # Format Access Group Dictionary
+        with open(access_groups, 'r') as file:
+            groups = ast.literal_eval(file.read())
+    else:
+        groups = access_groups
+
+    if isinstance(devices, str):
+        # Format Lanes Dictionary
+        with open(devices, 'r') as file:
+            lanes = ast.literal_eval(file.read())
+    else:
+        # Format Lanes Dictionary
+        lanes = dict()
+        for line in devices:
+            lanes[line['ID']] = line['Name']
 
     # Check for Exits in Dataset and Remove
-    if 2 in entries['Event_tcc_type']:
-        exits = entries.loc[entries['Event_tcc_type'] == 2]
-        entries = entries.drop(index=exits.index)
+    exits = entries.loc[entries['Event_tcc_type'] != '1']
+    entries = entries.drop(index=exits.index)
 
     # ENTRY ID
     entries['Entry ID'] = 'DE-' + entries['Event_id'].astype(str)
@@ -519,14 +572,20 @@ def entries_transforms(entries, ramps, access_path, lanes_path):
     entries['Lot Name'] = entries['Event_ph_num'].astype(str).map(carparks)
 
     # ACCESS GROUPS
-    entries['Access Group'] = 'Transient'
+    entries['Access Group'] = entries['Event_tic_iso'].map(groups)
 
-    for key in groups.keys():
-        entries.loc[entries['Event_ph_num'] == key, 'Access Group'] = entries['Tic_para2'].map(groups[key])
+    entries.loc[~entries['Event_tic_iso'].str.contains('EMI', na=False), 'Access Group'] = 'Transient'
 
-    # LANE
-    with open(lanes_path, 'r') as file:
-        lanes = ast.literal_eval(file.read())
+    entries.loc[entries['Access Group'].isna(), 'Access Group'] = None
+
+    if isinstance(lanes, str):
+        # LANE
+        with open(lanes, 'r') as file:
+            lanes = ast.literal_eval(file.read())
+    else:
+        lanes = dict()
+        for line in devices:
+            lanes[line['ID']] = line['Name']
 
     entries['Lane'] = entries['Tic_tcc_num'].map(lanes)
 
@@ -608,19 +667,22 @@ def entries_transforms(entries, ramps, access_path, lanes_path):
     return entries
 
 
-def exits_transforms(exits, ramps, access_path, lanes_path):
+def exits_transforms(exits, ramps, access_groups, devices):
     # Format Carpark Dictionary
-    carparks = {}
+    carparks = dict()
     for line in ramps:
         carparks[line['Number']] = line['Name']
 
     # Format Access Group Dictionary
-    with open(access_path, 'r') as file:
-        groups = ast.literal_eval(file.read())
+    if isinstance(access_groups, str):
+        with open(access_groups, 'r') as file:
+            groups = ast.literal_eval(file.read())
+    else:
+        groups = access_groups
 
     # Check for Entries in Dataset and Remove
     if 1 in exits['Event_tcc_type']:
-        entries = exits.loc[exits['Event_tcc_type'] == 1]
+        entries = exits.loc[exits['Event_tcc_type'] != '2']
         exits = exits.drop(index=entries.index)
 
     # EXIT ID
@@ -633,14 +695,20 @@ def exits_transforms(exits, ramps, access_path, lanes_path):
     exits['Lot Name'] = exits['Event_ph_num'].astype(str).map(carparks)
 
     # ACCESS GROUPS
-    exits['Access Group'] = 'Transient'
+    exits['Access Group'] = exits['Event_tic_iso'].map(groups)
 
-    for key in groups.keys():
-        exits.loc[exits['Event_ph_num'] == key, 'Access Group'] = exits['Tic_para2'].map(groups[key])
+    exits.loc[~exits['Event_tic_iso'].str.contains('EMI', na=False), 'Access Group'] = 'Transient'
+
+    exits.loc[exits['Access Group'].isna(), 'Access Group'] = None
 
     # LANE
-    with open(lanes_path, 'r') as file:
-        lanes = ast.literal_eval(file.read())
+    if isinstance(devices, str):
+        with open(devices, 'r') as file:
+            lanes = ast.literal_eval(file.read())
+    else:
+        lanes = dict()
+        for line in devices:
+            lanes[line['ID']] = line['Name']
 
     exits['Lane'] = exits['Tic_tcc_num'].map(lanes)
 
