@@ -6,7 +6,7 @@ from socrata import Socrata
 
 
 # RENAME RAMPS FUNCTION
-def rename_ramp(fourfour, column, current_name, replacement, update_config):
+def rename_ramp(fourfour, column, current_name, replacement, update_config, blanks=False, blank_names=None):
     # GET EXISTING DATA
     url = 'https://data.grandrapidsmi.gov/resource/' + \
           fourfour + \
@@ -21,6 +21,11 @@ def rename_ramp(fourfour, column, current_name, replacement, update_config):
     # RECODE VARIABLE
     df[column] = replacement
 
+    # ADD COLUMNS WITH BLANK VALUES
+    if blanks is True:
+        for column in blank_names:
+            df[column] = None
+
     # SOCRATA AUTH
     auth = Authorization(
         'data.grandrapidsmi.gov',
